/* Checks if the descent direction is computed correctly using the
   analytic solution; also checks if the algorithm converges
   to the right solution regardless of where it starts */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <cassert>

#include <dune/common/shared_ptr.hh>

#include <dune/tectonic/ellipticenergy.hh>

#include "test-gradient-method-helper.hh"

int main() {
  int const dim = 2;
  typedef Dune::EllipticEnergy<dim> Functional;
  typedef Functional::SmallMatrix SmallMatrix;
  typedef Functional::SmallVector SmallVector;

  SmallMatrix const A = { { 3, 1.5 }, { 1.5, 4 } };
  SmallVector const b = { 1, 2 };

  auto const f = Dune::make_shared<Dune::TrivialFunction const>();
  auto const phi = Dune::make_shared<Functional::NonlinearityType const>(f);
  Functional const J(A, b, phi);

  SmallVector solution = { 4.0 / 39.0, 6.0 / 13.0 }; // Analytic solution

  double ret1;
  {
    SmallVector start = b;
    start *= 17;

    SmallVector analytic_descent = b;
    A.mmv(start, analytic_descent);
    SmallVector numerical_descent;
    J.descentDirection(start, numerical_descent);
    assert(two_distance<dim>(numerical_descent, analytic_descent) < 1e-10);

    ret1 = functionTester(J, start, 6);
    assert(two_distance<dim>(start, solution) < 1e-5);
  }

  double ret2;
  {
    // Something random
    SmallVector start = { 279, -96 };

    SmallVector analytic_descent = b;
    A.mmv(start, analytic_descent);
    SmallVector numerical_descent;
    J.descentDirection(start, numerical_descent);
    assert(two_distance<dim>(numerical_descent, analytic_descent) < 1e-10);

    ret2 = functionTester(J, start, 25);
    assert(two_distance<dim>(start, solution) < 1e-6);
  }
  assert(std::abs(ret1 - ret2) < 1e-5);
}
